#include <windows.h>
#include <gl/gl.h>
#include "font.h"

Font* GlobalFont;

void InitFont()
{
	GlobalFont = new Font;
}

void ShutdownFont()
{
	delete GlobalFont;
}

Font* GetFont()
{
	return GlobalFont;
}

void Font::ShowText(int option)
{
	HideText = (option == 0);
}

Font::Font()
{
	HideText = 0;
	Letters = 0;
	CharSpace = 2;
	Setup(' ', '~');
}

Font::~Font()
{
	if (Letters)
		delete [] Letters;
}

void Font::Draw( float startx, float starty, char* text, int lettercount)
{

	if (HideText)
		return;
	glBlendFunc(GL_SRC_ALPHA,GL_ONE);

	glBindTexture(GL_TEXTURE_2D, Texture);
	glBegin(GL_QUADS);
	

	glColor4f(0.0f,0.0f,0.0f,0.0f);
	// provide black backing for text
	Point_2d size = GetSize(text, lettercount);
	glVertex3f(startx, starty,  0.0f);
	glVertex3f( startx+size.x, starty,  0.0f);	
	glVertex3f( startx+size.x, starty+size.y,  0.0f);	
	glVertex3f( startx, starty+size.y,  0.0f);


	glColor4f(1.0f,1.0f,1.0f, .75f);


	float x = startx;
	float y = starty;
	for (char* pos = text; (pos - text) < lettercount; pos++)
	{
		if (*pos >= StartChar && *pos <= EndChar)
			x+=(DrawLetter( x,y,*pos - StartChar)+CharSpace);
		else
		{
			if (*pos == '\t')
			{
				for (int count=0;count < 5; count++)
					x+= (DrawLetter(x,y, ' '-StartChar)+CharSpace);
			}
			else if (*pos == '\n')
			{
				x = startx;
				y -= LineHeight;
			}
		}
	}

	
	glEnd();
	glBlendFunc(GL_ONE,GL_ZERO);
	glBindTexture(GL_TEXTURE_2D, 0);
}

float Font::DrawLetter( float x, float y, int index)
{
	if ((index < 0))
		return 0;

	float width = Letters[index].width * Scale;
	float height = (float)LineHeight;

	glTexCoord2f(Letters[index].left, Letters[index].top); 
	glVertex3f(x, y,  0.0f);
	glTexCoord2f(Letters[index].right, Letters[index].top); 
	glVertex3f( x+width, y,  0.0f);	
	glTexCoord2f(Letters[index].right, Letters[index].bottom); 
	glVertex3f( x+width, y-height,  0.0f);	
	glTexCoord2f(Letters[index].left, Letters[index].bottom);
	glVertex3f( x, y-height,  0.0f);
	return width;
}

// returns text dimensions in pixels
Point_2d Font::GetSize( char* text, int lettercount)
{
	Point_2d p;
	float x, y, max_x;
	x = 0;
	y = 0;
	max_x = 0;

	for (char* pos = text; (pos - text) < lettercount; pos++)
	{
		if (*pos >= StartChar && *pos <= EndChar)
			x+=((Letters[*pos - StartChar].width * Scale)+CharSpace);
		else
		{
			if (*pos == '\t')
			{
				for (int count=0;count < 5; count++)
				{
					if (StartChar<=' ')
						x+= ((Letters[' '-StartChar ].width * Scale)+CharSpace);
				}
			}
			else if (*pos == '\n')
			{
				if (x > max_x)
					max_x = x;
				x = 0;
				y -= LineHeight;
			}
		}
	}
	
	if (x > max_x)
		max_x = x;

	p.x = max_x;
	p.y = y-LineHeight;

	return p;
}
	
int Font::GetLineHeight()	// returns line height in pixels
{
	return LineHeight;
}


void Font::Setup( char first, char last)
{
	StartChar = first;
	EndChar = last;

	int fontsize, bolded;
	fontsize = 12;
	bolded = 0;

    // Create a DC and a bitmap for the font
    HDC     hDC       = CreateCompatibleDC( NULL );
    SetMapMode( hDC, MM_TEXT );

    // Create a font.  By specifying ANTIALIASED_QUALITY, we might get an
    // antialiased font, but this is not guaranteed.
    
    DWORD dwBold;
	if (bolded == 1)
		dwBold = FW_BOLD;
	else
		dwBold = FW_NORMAL;

    HFONT hFont    = CreateFont( 
							-fontsize, 
							0,	// default width
							0,	// no escapement
							0,	// no orientation angle
							dwBold,	// font weight
							0,	// font italic
							FALSE,		// underline	
							FALSE,		// strikeout	
							DEFAULT_CHARSET,
							OUT_DEFAULT_PRECIS,
							CLIP_DEFAULT_PRECIS,
							ANTIALIASED_QUALITY,
							VARIABLE_PITCH, 
							"arial");

    SelectObject( hDC, hFont );

	// select texture width and height based on area taken up by font
	int texwidth, texheight;
	char c;
	char str[] = "x";
	int x;
    int y;
    SIZE size;

	// start with 128 and double a dimension
	// every time it gets to the end and doesn't have room
	texwidth = texheight = 128;	
	int doubleflag = 0;

	do {
		x = 0; 
		y = 0;
		if (doubleflag)
		{
			if (texwidth > texheight)
				texheight *= 2;
			else
				texwidth *= 2;
		}
		doubleflag = 0;
		for( c=StartChar; c<=EndChar; c++ )
		{
			str[0] = c;
			GetTextExtentPoint32( hDC, str, 1, &size );

			if( (x+size.cx) >= texwidth )
			{
				x  = 0;
				y += size.cy+1;
				
				if (y + size.cy > texheight)
					doubleflag = 1;
			}
			x += size.cx+1;
		}
	} while (doubleflag == 1);
	// done finding optimal texture size

    // Prepare to create a bitmap
    DWORD*      pBitmapBits;
    BITMAPINFO bmi;
    memset( &bmi.bmiHeader, 0, sizeof(BITMAPINFOHEADER) );
    bmi.bmiHeader.biSize        = sizeof(BITMAPINFOHEADER);
    bmi.bmiHeader.biWidth       =  (int)texwidth;
    bmi.bmiHeader.biHeight      = -(int)texheight;
    bmi.bmiHeader.biPlanes      = 1;
    bmi.bmiHeader.biCompression = BI_RGB;
    bmi.bmiHeader.biBitCount    = 24;

    HBITMAP hbmBitmap = CreateDIBSection( hDC, &bmi, DIB_RGB_COLORS,
                                          (VOID**)&pBitmapBits, NULL, 0 );

    SelectObject( hDC, hbmBitmap );

    // Set text properties
    SetTextColor( hDC, RGB(255,255,255) );
    SetBkColor(   hDC, 0x00000000 );
    SetTextAlign( hDC, TA_TOP );

    // Loop through all printable character and output them to the bitmap..
    // Meanwhile, keep track of the corresponding tex coords for each character.

	CharInfo *info = new CharInfo[ (EndChar - StartChar) + 1];
	x = 0;
	y = 0;

    for( c=StartChar; c<=EndChar; c++ )
    {
        str[0] = c;
        GetTextExtentPoint32( hDC, str, 1, &size );

        if( x+size.cx >= texwidth )
        {
            x  = 0;
            y += size.cy+1;
        }

        ExtTextOut( hDC, x, y, ETO_OPAQUE, NULL, str, 1, NULL );

        info[c-StartChar].left = ((float)x)/(float)texwidth;
		info[c-StartChar].top = ((float)y)/(float)texheight;
		info[c-StartChar].right = (float)(x+size.cx)/(float)texwidth;
		info[c-StartChar].bottom = (float)(y+size.cy)/(float)texheight;
		info[c-StartChar].width = info[c-StartChar].right-info[c-StartChar].left;

		LineHeight = size.cy+1;
	    x += size.cx+1;
    }

	int bpp = 3;

	glGenTextures(1, &Texture);
	glBindTexture(GL_TEXTURE_2D, Texture);

	// type, mips, bpp, width, height, format, bytetype, data
	glTexImage2D(GL_TEXTURE_2D, 0, 3, texwidth, texheight, 0, GL_RGB, GL_UNSIGNED_BYTE, pBitmapBits);

	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST );	
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_NEAREST);	

	Scale = (float)texwidth;
	Letters = info;
    DeleteObject( hbmBitmap );
    DeleteDC( hDC );
    DeleteObject( hFont );

	glBindTexture(GL_TEXTURE_2D, 0);

}
