#include "gui.h"
#include "font.h"

void DrawButton(float left, float top,
				float width, float height,
				char* text, 
				float innercolor[3], float outercolor[3] )
{
	// draw a button
	// made up of 5 quads
	// outside color for each outside vertex, 
	// inside color for inner vertices
	// inner square is 10% smaller than outer
	// ----------
	// |\______/|
	// | |     ||
	// | ------ |
	// |/______\|
	float partwidth, partheight;
	partwidth = 5;//width/10;
	partheight = 3;//height/10;


	glBegin(GL_QUADS);

	// mid quad
	glColor3f(innercolor[0],innercolor[1],innercolor[2]);
	glVertex3f(left+partwidth,top-partheight,0);
	glVertex3f(left+width-partwidth,top-partheight,0);
	glVertex3f(left+width-partwidth,top-height+partheight,0);
	glVertex3f(left+partwidth,top-height+partheight,0);

	// top quad
	// top left
	glColor3f(outercolor[0],outercolor[1],outercolor[2]);
	glVertex3f(left,top,0);
	// top right
	glVertex3f(left+width,top,0);
	// bottom right
	glColor3f(innercolor[0],innercolor[1],innercolor[2]);
	glVertex3f(left+width-partwidth, top-partheight,0);
	// bottom left
	glVertex3f(left+partwidth,top-partheight,0);
	
	// right quad	
	// top right
	glColor3f(outercolor[0],outercolor[1],outercolor[2]);
	glVertex3f(left+width,top,0);
	// bottom right
	glVertex3f(left+width,top-height,0);
	// bottom left
	glColor3f(innercolor[0],innercolor[1],innercolor[2]);
	glVertex3f(left+width-partwidth, top-height+partheight,0);
	// top left
	glVertex3f(left+width-partwidth,top-partheight,0);
	
	// bottom quad
	// top left
	glColor3f(outercolor[0],outercolor[1],outercolor[2]);
	glVertex3f(left,top-height,0);
	// top right
	glVertex3f(left+width,top-height,0);
	// bottom right
	glColor3f(innercolor[0],innercolor[1],innercolor[2]);
	glVertex3f(left+width-partwidth, top-height+partheight,0);
	// bottom left
	glVertex3f(left+partwidth,top-height+partheight,0);

	// left quad	
	// top right
	glColor3f(outercolor[0],outercolor[1],outercolor[2]);
	glVertex3f(left,top,0);
	// bottom right
	glVertex3f(left,top-height,0);
	// bottom left
	glColor3f(innercolor[0],innercolor[1],innercolor[2]);
	glVertex3f(left+partwidth, top-height+partheight,0);
	// top left
	glVertex3f(left+partwidth,top-partheight,0);

	glEnd();	

	// check if extends off of button
	int lettercount = strlen(text);
	int lettershown = lettercount;
	
	Point_2d dim = GetFont()->GetSize(text, lettershown);
	// bias 4 pixels, in case only one letter is left
	if ( (dim.x-4) > (width-(2*partwidth)) )
	{
		// does not fit, so crop text
		
		while (dim.x > (width-(2*partwidth)))
		{
			lettershown--;
			dim = GetFont()->GetSize(text, lettershown);
		}
	}

	GetFont()->Draw( left+partwidth, top-partheight, text, lettershown);
	if (lettershown != lettercount)
	{
		GetFont()->Draw( left+partwidth+dim.x, top-partheight, "..", 2);
	}

}

int IntersectButton( float checkx, float checky,
					 float left, float top,
					 float width, float height)
{
	if ( (left <= checkx ) && (checkx <= (left+width)))
	{
		if ( (top >= checky) && (checky >= top-height))
			return 1;
	}
	return 0;
}


GUILayer::GUILayer()
{
	SubMultiplier = 1.5;
	CurrentOption = 0;
	TabWidth = 120;
	BarHeight = 20;
	Active = 0;
	Offset = 5;
}

// adds an option to the main menu
void GUILayer::AddMenuOption( char* name)
{
	MenuOption toadd;
	toadd.Name = (std::string)name;
	Options.push_back(toadd);
}

// adds an item to a menu option
// parent is name of menu option
void GUILayer::AddMenuItem( char* parent, char* name, GUIListener* listener, int startcolor)
{
	// check to see which has correct name
	MenuOption* found = 0;
	for (std::list<MenuOption>::iterator i = Options.begin(); (i != Options.end()) && (found == 0); i++)
	{
		const char* stt = i->Name.c_str();
		if (i->Name.compare(parent) == 0)
			found = &(*i);
	}

	if (found==0)
		return;

	MenuItem toadd;
	toadd.Name = (std::string)name;
	toadd.Listener = listener;
	toadd.ColorOption = startcolor;
	found->Children.push_back(toadd);
	listener->menu = &found->Children.back();
}

int GUILayer::OnMouse( int x, int y, int buttons[4], int buttonchange)
{
	if (!Active)
		return 0;
	int ResetTab = 0;	// set if tab should be closed after click
	Point_2d res = GetSceneGraph()->GetResolution();

	if ((buttonchange==0)&&(buttons[0] ==1))
	{
		// clicked, so test menu region
		if (y > (res.y - BarHeight))
		{
			for (std::list<MenuOption>::iterator i = Options.begin(); i != Options.end(); i++)
			{
				if (IntersectButton( x, y,
					 i->x, i->y,
					 TabWidth, BarHeight))
				{
					if (CurrentOption == &(*i))
						CurrentOption = 0;
					else
						CurrentOption = &(*i);
				}
			}
		}
		else if (CurrentOption != 0)
		{	
			// if there is a current menu tab, check its children
			for (std::list<MenuItem>::iterator c = CurrentOption->Children.begin(); c != CurrentOption->Children.end(); c++)
			{
				if (IntersectButton( x, y,
					 c->x, c->y,
					 TabWidth*SubMultiplier, BarHeight))
				{
					c->ColorOption = c->Listener->OnClick();
					ResetTab = 1;
				}
			}
		}
	}

	if (ResetTab)
		CurrentOption = 0;

	return 0;
}

int GUILayer::Draw( )
{
	if (!Active)
		return 0;

	Point_2d res = GetSceneGraph()->GetResolution();

	float top = res.y;
	// subdivide top of screen among menu listeners
	float x = 0.0f;

	float outercolor[3] = { .1f, .1f, .1f};
	float standard[3] = { .1f,.4f, .6f};
	float special[3] = { .1f, .1f, .6f};
	float inner[3];


	for (std::list<MenuOption>::iterator i = Options.begin(); i != Options.end(); i++)
	{	
		if (&(*i) == CurrentOption)
			memcpy( inner, special, sizeof(special));
		else
			memcpy( inner, standard, sizeof(standard));

		// draw a button for the main option
		i->x = x;
		i->y = top;
		DrawButton(x, top,
				TabWidth, BarHeight,
				(char*)i->Name.c_str(), 
				inner, outercolor );

		// if main option is active, draw its children too
		if (&(*i) == CurrentOption)
		{
			float childleft = x + Offset;
			float childwidth = TabWidth * SubMultiplier;
			float childtop = top - BarHeight;

			for (std::list<MenuItem>::iterator c = i->Children.begin(); c != i->Children.end(); c++)
			{
				if (c->ColorOption == 1)
					memcpy( inner, special, sizeof(special));
				else
					memcpy( inner, standard, sizeof(standard));

				c->x = childleft;
				c->y = childtop;
				DrawButton( childleft, childtop,
							childwidth, BarHeight,
							(char*)c->Name.c_str(),
							inner, outercolor);
				childtop-=BarHeight;
			}
		}
		x+= TabWidth;
	}
	
	 

	return 0;
}

void GUILayer::SetActive( int truefalse)
{
	Active = truefalse;
}

void GUILayer::RemoveMenuOption( char* name)
{
	// check to see which has correct name
	for (std::list<MenuOption>::iterator i = Options.begin(); i != Options.end(); i++)
	{
		const char* stt = i->Name.c_str();
		if (i->Name.compare(name) == 0)
		{
			Options.erase(i);
			return;
		}
	}


	
}
