#include <windows.h>
#include <stdlib.h>

#include "font.h"
#include "scenegraph.h"
#include "particle.h"
#include "potential.h"
#include "field.h"
#include "gui.h"

class ScreenSaverListener : public SceneLayer
{
public:
	int OnKey( SDL_KeyboardEvent* key) { GetSceneGraph()->QuitApp(); return 1; }
	int GetActive() { return 1; }
	int OnMouse( int x, int y, int buttons[4], int buttonchange) 
	{ 
		GetSceneGraph()->QuitApp(); 
		return 1; 
	}
};

class Boundary : public ParticleListener, public GUIListener
{
public:
	int state;
	ParticleLayer* particles; 
	Boundary( ParticleLayer* parent, int s) 
	{ 
		particles = parent; 
		state = s;
		if (state)
			particles->AddListener(this);
	}
	int OnClick() 
	{
		if (state)
		{
			particles->RemoveListener(this);
			state = 0;

		}
		else
		{
			particles->AddListener(this);
			state = 1;
		}
		return state;
	}
	void OnMove( ParticleState* state)
	{
		Point_2d r = GetSceneGraph()->GetResolution();
		// edge checking
		if (state->pos.x > r.x)
		{
			state->velocity.x = -state->velocity.x;
			state->pos.x = r.x;
		}
		if (state->pos.y > r.y)
		{
			state->velocity.y = -state->velocity.y;
			state->pos.y = r.y;
		}
		if (state->pos.x < 0)
		{
			state->velocity.x = -state->velocity.x;
			state->pos.x = 0;
		}
		if (state->pos.y < 0)
		{
			state->velocity.y = -state->velocity.y;
			state->pos.y = 0;
		}
	}


};


class LayerMenuListener : public GUIListener
{
public:
	int OnClick() { return 1; }
};

class FieldToggle : public GUIListener
{
public:
	FieldToggle( SceneLayer* layer, int startstate) { parent = layer; }
	int OnClick() 
	{
		int state = (parent->GetActive()+1)&0x01;
		parent->SetActive(state);
		return 0;
	}

	SceneLayer* parent;
};

void PlaceRandom( ParticleLayer* parent)
{
	Point_2d r = GetSceneGraph()->GetResolution();
	ParticleState p;
	p.start.x = rand() % (int)r.x;
	p.start.y = rand() % (int)r.y;
	p.mass = .051;
	//p.fixed = 1;//rand()%2;
	int sign = rand()%2;
	if (sign == 0)
		sign = -1;
	p.charge = sign*200;
	parent->AddParticle( p);
}


class Random : public GUIListener
{
public:
	Random(ParticleLayer* p) { particles = p; }
	ParticleLayer* particles;
	int OnClick()
	{
		PlaceRandom(particles);
		return 0;
	}
};

class ClearParticles : public GUIListener
{
public:
	ClearParticles( ParticleLayer* p) { particles = p; }
	ParticleLayer* particles;
	int OnClick()
	{
		particles->Clear();
		return 0;
	};
};

class ResetStart : public GUIListener
{
public:
	ResetStart( ParticleLayer* p) { particles = p; }
	int OnClick()
	{
		for (int x = 0; x < particles->GetTotalParticles(); x++)
		{
			ParticleState* state = particles->GetParticle(x);
			state->velocity = Point_2d(0,0);
			state->accel = Point_2d(0,0);
			state->pos = state->start;
			
		}
		return 0;
	}
	ParticleLayer* particles;
};

class TrailToggle : public GUIListener
{
public:
	ParticleLayer* parent;
	int state;
	TrailToggle(ParticleLayer* p) { parent = p; state = 0; }
	int OnClick()
	{
		state = (state+1) &0x01;
		parent->EnableTrails(state*25);
		return 0;
	}
};

class DrawParticlesToggle : public GUIListener
{
public:
	ParticleLayer* parent;
	int state;
	DrawParticlesToggle( ParticleLayer* p) { parent =p; state = 1; }
	int OnClick()
	{
		state = (state+1) & 0x01;
		parent->SetDrawParticles(state);
		return 0;
	}
};

int __stdcall WinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nShowCmd)
{
	ScreenSaverListener screensaverquit;
	int width, height, fullscreen, screensaverenabled, 
		potentialstate, fieldstate, guistate, boundarystate;
	width = 640;
	height = 480;
	fullscreen = screensaverenabled = 0;
	potentialstate = fieldstate = 0;
	guistate = boundarystate = 1;
	
	// command line parsing
	// check for resolution, screensaver, etc, default field state
	if( lpCmdLine)
	{
		// goes through and checks if string contains tag
		// assigns position at same time as check
		char* pos;
		// check for resolution tag
		if (pos = strstr( lpCmdLine, "-resolution") )
		{
			pos+= strlen("-resolution");
			width = strtol( pos, &pos, 10);
			height = strtol( pos, &pos, 10);
		}
		if (pos = strstr( lpCmdLine, "-screensaver"))
		{
			// if the screen saver is turned on, it means
			// the placement layer is turned off, and should add
			// a mouse/key listener which turn off app
			// set the app to fullscreen
			fullscreen = 1;
			screensaverenabled = 1;
			fieldstate = potentialstate = 1;
			guistate = 0;
		}
	}

	InitSceneGraph(width, height, fullscreen);
	InitFont();
	SceneGraph *app = GetSceneGraph();
	app->Run();	// clear starting messages
	if (screensaverenabled)
		app->AddLayer(&screensaverquit, "screensaverquit");

	ParticleLayer particles;
	app->AddLayer(&particles, "particles");

	PotentialLayer potential(&particles);
	app->AddLayer(&potential, "potential", -1);
	if (potentialstate)
		potential.SetActive(1);

	FieldLayer field(&particles);
	app->AddLayer(&field, "electric field");
	if (fieldstate)
		field.SetActive(1);
	
	TrailToggle trail(&particles);
	DrawParticlesToggle drawparticles(&particles);

	if (screensaverenabled)
	{
		// add several random particles for the viewers pleasure
		for (int x = 0; x < 10; x++)
			PlaceRandom( &particles);
		SDL_ShowCursor(0);
		GetFont()->ShowText(0);
	}

	/////////////////////////////////////////////////
	// SETUP GUI
	GUILayer gui;
	app->AddLayer(&gui, "gui",20);
	if (guistate)
		gui.SetActive(1);
	
	FieldToggle potentialtoggle(&potential, potentialstate);
	FieldToggle fieldtoggle(&field, fieldstate);
	Boundary boundarytoggle(&particles, boundarystate);
	Random random(&particles);
	ResetStart reset(&particles);
	ClearParticles clear(&particles);

	LayerMenuListener layermenu;

	gui.AddMenuOption("Edit");
		gui.AddMenuItem("Edit", "Reset Points to Start", &reset,0);
		gui.AddMenuItem("Edit", "Place Random Particle", &random,0);
		gui.AddMenuItem("Edit", "Clear Particles", &clear,0);
	
	gui.AddMenuOption("Visualization");
		gui.AddMenuItem("Visualization", "Potential On/Off", &potentialtoggle,0);
		gui.AddMenuItem("Visualization", "Electric Field On/Off", &fieldtoggle, 0);
		gui.AddMenuItem("Visualization", "Trail Toggle", &trail, 0);
		gui.AddMenuItem("Visualization", "Draw Particles On/Off", &drawparticles, 0);

	/////////////////////////////////////////////////

	
	while (app->Run())
	{

	}

	ShutdownFont();
	ShutdownSceneGraph();

	return 0;
}