	

#include "particle.h"

int DrawParticle( float x, float y, int charge)
{
	float particlesize = 10;
	float half = particlesize / 2;
	float r,g,b;
	r = g = b = 0;
	if (charge<0)
		r = 0xFF;
	else if (charge>0)
		g = 0xFF00;

	glBegin(GL_QUADS);								
	glColor3f (r, g, b);						
	
	// diamond pattern
	glVertex3f( half+x, half+y, 0.0f);				
	glVertex3f(-half+x,half+y, 0);				
	glVertex3f(-half+x,-half+y, 0);				
	glVertex3f(half+x,-half+y, 0);				
	glEnd();
	
	glBegin(GL_LINES);
	glColor3f( 0,0,0);
	// draw a plus or minus depending on charge
	if (charge<0)
	{
		glVertex3f(-4+x,y, 0);				
		glVertex3f(4+x,y, 0);				
	}
	else if (charge>0)
	{
		glVertex3f(-4+x,y, 0);				
		glVertex3f(4+x,y, 0);				
		glVertex3f(x,-4+y, 0);				
		glVertex3f(x,4+y, 0);				

	}
	glEnd();

	return 1;
}

int DrawForce( Point_2d pos, Point_2d force, float intensity)
{
	// draw a colored line starting at pos
	// ending at pos + force
	// with an arrowhead

	force = force*3;
	if (fabs(force.x) < 5 && fabs(force.y) < 5)
	{
		force.Normalize();
		force *= 5;
	}
	glBegin(GL_LINES);
	glColor3f(intensity,intensity,intensity);
	glVertex3f( pos.x, pos.y, 0);
	glVertex3f( pos.x+force.x, pos.y+force.y, 0);

	Point_2d invslope;
	invslope.x = -force.y/6;
	invslope.y = force.x/6;
	Point_2d s, p1, p2;
	s = force*.5;
	p1 = s + invslope;
	p2 = s - invslope;
	
	glVertex3f( pos.x+p1.x, pos.y+p1.y, 0);
	glVertex3f( pos.x+force.x, pos.y+force.y, 0);
	glVertex3f( pos.x+p2.x, pos.y+p2.y, 0);
	glVertex3f( pos.x+force.x, pos.y+force.y, 0);


	glEnd();
	return 1;
}




ParticleLayer::ParticleLayer()
{
	TrailSize = 0;
	Active = 1;
	EnableDrawParticles = 1;
}

ParticleLayer::~ParticleLayer()
{
	Listeners.resize(0);
	for(int x = 0; x < Particles.size(); x++)
		delete Particles[x];
	Particles.resize(0);
}

void ParticleLayer::AddListener( ParticleListener* p)
{
	Listeners.push_back(p);
}

void ParticleLayer::RemoveListener( ParticleListener* p)
{
	Listeners.remove(p);
}

void ParticleLayer::Clear()
{
	while (GetTotalParticles())
		RemoveParticle((int)0);
}

void ParticleLayer::AddParticle( ParticleState p)
{
	ParticleState *toadd;
	toadd = new ParticleState;
	*toadd = p;
	toadd->pos = toadd->start;

	
	Particles.push_back(toadd);
	Trails.resize(Particles.size());
}

void ParticleLayer::RemoveParticle( ParticleState* p)
{
	for (int x = 0; x < Particles.size(); x++)
	{
		if (Particles[x] == p)
		{
			RemoveParticle(x);
			return;
		}
	}
}

void ParticleLayer::RemoveParticle(int index)
{
	if (index > Particles.size() || Particles.size() == 0)
		return;
	delete Particles[index];
	// swap last particle with current particle
	
	if (Particles.size() > 1)
	{
		ParticleState *last;
		last = Particles[Particles.size()-1];
		Particles[index] = last;
	}
	Particles.resize( Particles.size()-1);

	// reset trails, don't want to go through hassle of copying
	Trails.clear();
	Trails.resize( Particles.size());
}



int ParticleLayer::GetTotalParticles()
{
	return Particles.size();
}

ParticleState* ParticleLayer::GetParticle(int index)
{
	return Particles[index];
}

int ParticleLayer::OnMouse( int x, int y, int buttons[4])
{
	// does not handle mouse input
	return 0;
}

int ParticleLayer::Draw( )
{
	// draws all particles at position
	if (EnableDrawParticles)
	{
		for (std::vector<ParticleState*>::iterator i = Particles.begin(); i != Particles.end(); i++)
		{
			
			if ((*i)->visible)
				DrawParticle( (*i)->pos.x, (*i)->pos.y, (*i)->charge);
		}
	}

	// if trails are enabled, draw them
	if (TrailSize && (Trails.size() > 1))
	{
		for (int x = 0; x < Trails.size(); x++)
		{
			if (Trails[x].size())
			{
				float pos = 0;
				for (std::deque<TrailInfo>::iterator t = Trails[x].begin(); t != Trails[x].end(); t++, pos++)
				{
					// draw with intensity relative to age of item in list
					DrawForce(t->pos, t->force, (pos / (Trails[x].size())));
				}
			}
		}

	}

	return 0;
}

// update the elements in the layer
int ParticleLayer::Tick(float timedelta)
{

	// insert code here to perform multiple iterations
	// moveparticles returns 0 if stable
	// returns stable smaller time than timedelta if instable
	// if returns instable time, iterate several times
	
	float timecomputed = 0.0f;
	while (timecomputed < timedelta)
	{
		timecomputed += MoveParticles(timedelta-timecomputed);
	}



	return 0;
}

float ParticleLayer::MoveParticles(float timedelta)
{
	float timesimulated = timedelta;
	// updates all mobile particle positions
	// sends out particle moved messages
	
	int pos = 0;
	for (std::vector<ParticleState*>::iterator currenti = Particles.begin(); currenti != Particles.end(); currenti++, pos++)
	{
		ParticleState* current = *currenti;
		// move mobile particles
		if (current->fixed != 1)
		{
		
			Point_2d force(0,0);
			force = CalcForce( current);
			// base on the mass apply acceleration
			// a = (f/m)
			Point_2d acceleration;
			acceleration = force / (current->mass);
			
			current->accel.x = acceleration.x;
			current->accel.y = acceleration.y;

			// clamp maximum acceleration, and tesselate time
			// for stability
			if (current->accel.Magnitude() > 300.0f)
			{
				current->accel.Normalize(); 
				current->accel *= 300.0f;
				
				if( timesimulated > 0.005f)
					timesimulated = 0.005f;
			}
			// if trails are enabled, add to the trail
			if (TrailSize)
			{
				TrailInfo t;
				t.pos = current->pos;
				t.force = force;
				// only record if different than last
				if (Trails[pos].size() == 0)
				{
					Trails[pos].push_back(t);
				}
				else if (Point_2d::Distance( (Trails[pos])[Trails[pos].size()-1].pos, current->pos) > 10)
				{
					Trails[pos].push_back(t);
					if (Trails[pos].size() > TrailSize)
						Trails[pos].pop_front();
				}
			}
		}

	}

	for (std::vector<ParticleState*>::iterator movei = Particles.begin(); movei != Particles.end(); movei++)
	{
		ParticleState* move = *movei;
		if (move->fixed != 1)
		{

			move->velocity.x += move->accel.x * timesimulated;
			move->velocity.y += move->accel.y * timesimulated;

			move->pos.x += move->velocity.x  * timesimulated;
			move->pos.y += move->velocity.y * timesimulated;

			for (std::list<ParticleListener*>::iterator i = Listeners.begin(); i != Listeners.end(); i++)
			{
				(*i)->OnMove( &(*move));
			}
		}
	}
	return timesimulated;
}

// calculates force exerted on p1 by p2
Point_2d ParticleLayer::CalcForce(ParticleState* current)
{
	Point_2d force(0,0);
	// calculate force vector based on all other particles
	for (std::vector<ParticleState*>::iterator influencei = Particles.begin(); influencei != Particles.end(); influencei++)
	{
		ParticleState* influence = *influencei;
		if (influence != current)
		{
			Point_2d difference;
			if (current->charge == 0 || influence->charge == 0)
			{
				difference=Point_2d(0,0);
				goto end;
			}

			double magnitude;
			// force = k(q1q2)/r^2
			// however this is unitless so k is omitted

			double radius;
			difference = influence->pos - current->pos;

			if (difference == Point_2d(0,0))
			{
				// cannot do anything if on top of eachother;
				goto end;
			}

			radius = difference.Magnitude();

			// clamp radius, small radii generate very large forces
			if (radius < 3)
				radius = 3;

			magnitude = (current->charge * influence->charge)/ (radius*radius);
			magnitude = fabs(magnitude);

			// point in correct direction, scale by force, then calculate sign
			difference.Normalize();
			difference*=magnitude;
			
			int sign1, sign2;
			sign1 = (current->charge > 0);
			sign2 = (influence->charge > 0);

			if (sign1 == sign2)
				difference *= -1;	// like repel
			
			end:
			force += difference;
		}
	}
	return force;
}