#ifndef PARTICLEINC
#define PARTICLEINC

#pragma warning (disable: 4786)

#include <vector>
#include <deque>
#include "scenegraph.h"
#include "point2d.h"

// draws a particle
// x, y indicate center

int DrawParticle( float x, float y, int charge);
int DrawForce( Point_2d pos, Point_2d force, float intensity);
class ParticleState
{
public:
	ParticleState()
	{
		charge = mass = fixed = 0;
		velocity.x = velocity.y = 0;
		pos = start = velocity;
		visible = 1;
	}
	Point_2d pos;				// current position
	Point_2d start;	// starting position
	int id;					// can be set when created, for use by others
	int charge;				// positive or negative charge with magnitude
	double mass;				// mass in units
	Point_2d velocity;		// current velocity in pixels per second
	Point_2d accel;
	int visible;
	int fixed;				// boolean value for whether fixed or not
};

class ParticleListener
{
public:
	virtual void OnMove( ParticleState* state) = 0;
};

class ParticleLayer : public SceneLayer
{
public:
	ParticleLayer();
	~ParticleLayer();

	void AddParticle( ParticleState p);
	void RemoveParticle(int index);
	void RemoveParticle( ParticleState* p);
	int GetTotalParticles();
	ParticleState* GetParticle(int index);

	void AddListener( ParticleListener* p);
	void RemoveListener( ParticleListener* p);

	int OnMouse( int x, int y, int buttons[4]);
	int Draw( );
	// update the elements in the layer
	int Tick(float seconds);

	void Clear();
	// calculates force exerted on p1 by p2
	Point_2d CalcForce(ParticleState* p);

	void SetActive() {Active = 0; }
	int GetActive() { return Active; }
	
	void SetDrawParticles( int option) { EnableDrawParticles = option; }
	// set to enable trails
	// size indicates how many intervals trail will be saved over
	// set to 0 for no trails
	void EnableTrails(int size) { TrailSize = size; }
private:
	struct TrailInfo
	{		
		Point_2d pos;
		Point_2d force;
	};	
	int EnableDrawParticles;
	int TrailSize;
	int Active;
	float MoveParticles(float delta);
	std::vector<ParticleState*> Particles;
	std::list<ParticleListener*> Listeners;
	typedef std::deque<TrailInfo> TrailVector;
	std::vector< TrailVector > Trails;
};

#endif