#include "potential.h"

struct Color
{
	float r, g, b;
};

PotentialLayer::PotentialLayer (ParticleLayer* p)
{ 
	Active = 0;
	Particles = p; 
	Multiplier = .4f;
}

int PotentialLayer::Draw( )
{ 
	if (!Active)
		return 0;

	if (Particles->GetTotalParticles() == 0)
		return 0;
	SDL_Rect rect;
	int divisor = 8;
	rect.w = divisor;
	rect.h = divisor;

	Point_2d res = GetSceneGraph()->GetResolution();

	Color* mem = new Color[res.x/divisor+1];
	Color* mem2 = new Color[res.x/divisor+1];

	Color *newline = mem;
	Color *oldline = mem2;
	glBegin(GL_QUADS);	

	int start = 1;



	for(int y = 0; y <= res.y; y+=divisor)
	{
		for ( int x = 0; x <= res.x; x+=divisor)
		{
			rect.x = x;
			rect.y = y;
			float result = CalcField( Point_2d(x,y));
			
			// red is max negative
			// blue is max positive
			// all are clamped

			Color c;
			c.r = c.g = c.b = 0;
			
			// due to nonlinear gamma ramp on screen
			// sqrt gives more accurate results

			if (result > 0)
			{
				c.r = sqrt(result) * Multiplier;
				if (c.r>=1)
				{
					c.b = c.r - 1;
					if (c.b >1)
					{
						c.b = 1;
					}

					c.r = 1;
				}
			}
			else
			{
				c.g = sqrt(-result) * Multiplier;
				if (c.g >= 1)
				{
					c.b = c.g - 1;
					if (c.b > 1)
					{
						c.b = 1;
					}
					c.g = 1;
				}
			}
					
			newline[ x/divisor ] = c;

		}

		// after colors are all filled, draw
		// cannot draw first line due to fact that needs
		// previous line to draw
		if (start == 0)
		{
			for (int i = divisor; i <= res.x; i+=divisor)
			{
				// draw quad connecting previous to left
				// and top
				Color* c1, *c2, *c3, *c4;
				
				c1 = &oldline[(i-1)/divisor];
				c2 = &oldline[i/divisor];
				
				c3 = &newline[i/divisor];
				c4 = &newline[(i-1)/divisor];
				
				// top left
				glColor3f (c1->r, c1->g,c1->b);						
				glVertex3f(i-divisor,y-divisor, 0.0f);				

				// top right
				glColor3f (c2->r, c2->g,c2->b);						
				glVertex3f(i,y-divisor, 0);				

				// bottom right
				glColor3f (c3->r, c3->g,c3->b);						
				glVertex3f(i,y, 0);				

				// bottom left
				glColor3f (c4->r, c4->g,c4->b);						
				glVertex3f(i-divisor,y, 0);				

			}

		}
		else
			start = 0;

		Color *templine;
		templine = oldline;
		oldline = newline;
		newline = templine;
	}

	
	glEnd();
	delete [] mem;
	delete [] mem2;

	return 0; 
}

void PotentialLayer::SetActive(int active)
{ 
	Active = active; 
	if (Active)
	{
		GetSceneGraph()->AddNote("Potential visualization activated \nPlaces of equal color have equal potential\nColor does not indicate absolute strength\nplease note potential is calculated as if charges were static",15);
	}
}

float PotentialLayer::CalcField( Point_2d& pos)
{
	float field = 0.0f;
	for (int x = 0; x < Particles->GetTotalParticles(); x++)
	{
		ParticleState *current = Particles->GetParticle(x);
		float distance;
		distance = Point_2d::Distance(pos, current->pos);
		field += current->charge / (distance);
	}

	return field;
}