#ifndef SCENEGRAPHINC
#define SCENEGRAPHINC

#include <windows.h>
#include <gl\gl.h>		
#include <gl\glu.h>		
#include <gl\glaux.h>
#include <list>
#include <string.h>
#include "sdl.h"

#include "point2d.h"

class SceneLayer
{
public:
	// buttonchange is -1 if no change, otherwise button index if change
	virtual int OnMouse( int x, int y, int buttons[4], int buttonchange) { return 0; }
	virtual int OnKey( SDL_KeyboardEvent* key) { return 0; }
	virtual int Draw( ) { return 0; }
	virtual void SetActive( int truefalse) { }
	virtual int GetActive() = 0;
	// update the elements in the layer
	virtual int Tick(float delta_seconds) { return 0; }
	virtual void Shutdown() { }

// private
	std::string Name;
	int Precedence;
};

class SceneGraph
{
public:
	SceneGraph();
	~SceneGraph();

	int Initialize(int width, int height, int fullscreen);
	void Shutdown();

	void AddLayer(SceneLayer* layer, char* name, int order = 0);
	void RemoveLayer(SceneLayer* layer);
	SceneLayer* GetLayer( char* name);

	// changes time scale
	void SetTimeScale( float scale);
	void StopTimer();
	void StartTimer();

	// displays text for displaytime seconds
	void AddNote( char* text, float displaytime);
	void ClearNotes();
	void QuitApp();
	int Run();

	Point_2d GetResolution();
private:
	struct NoteInfo
	{
		std::string Text;
		float DisplayTime;
	};
	int Initialized;
	int MouseX, MouseY;
	float TimeScale;
	float MaxInterval;
	int SDLFlags;
	int BPP;
	int TimeActive;
	int MouseButtons[4];
	void PumpMessages();
	Point_2d Resolution;
	std::list<SceneLayer*> Layers;
	// due to fact cannot remove while traversing lists
	// there is a pending to remove list
	std::list<SceneLayer*> ToRemove;
	void ClearRemove();	// iterate through toremove and clear
	std::list<NoteInfo> Notes;

	SDL_Surface *Screen;

	int Active;
};

void InitSceneGraph(int width, int height, int fullscreen);
void ShutdownSceneGraph();
SceneGraph* GetSceneGraph();



#endif